==> mutual_credit-5.0.x-dev.tar.gz: mutual_credit/src/EventSubscriber/TransactionSubscriber.php <==
<?php

namespace Drupal\mcapi\EventSubscriber;

use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\Event\McapiEvents;
use Drupal\mcapi\Event\TransactionSaveEvent;
use Drupal\mcapi\Event\TransactionAssembleEvent;
use Drupal\user\Entity\User;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Config;
use Drupal\Core\Session\AccountInterface;
use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Example subscriber showing the two McapiEvents.
 * @todo inject State
 */
class TransactionSubscriber implements EventSubscriberInterface {

  protected Config $mcapiSettings;
  protected int $currentUserId;

  function __construct(ConfigFactoryInterface $config_factory, AccountInterface $account) {
    $this->mcapiSettings = $config_factory->get('mcapi.settings');
    $this->currentUserId = $account->id();
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() : array{
    return [
      #McapiEvents::ASSEMBLE => ['makeChildren'],
      McapiEvents::ACTION => ['onTransactionAction']
    ];
  }

  /**
   * Example
   *
   * Payer pays a fee of 2 raw units to the wallet 1.
   *
   * @param TransactionAssembleEvent $event
   *   $event->getSubject() gives the transaction.
   * @param string $eventName
   *   Which is always mcapi_transaction.assemble.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   */
  public function makeChildren(TransactionAssembleEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    $original_transaction = $event->getSubject();
    $config = \Drupal::config('system.site');
    $original_transaction->createChild([
      'payer' => $original_transaction->payer->target_id,
      'payee' => 1,
      'quantity' => 2, // note this is the raw value. Same currency unless stated otherwise.
      'description' => $this->t('Payer fee to @wallet', ['@wallet' => Wallet::load(1)->name->value])
    ]);
  }

  /**
   * Acts on a transaction and returns a render array in $events->output.
   *
   * @param TransactionSaveEvent $event
   *   $event->getSubject() gives the transaction.
   *   $event->getArguments() yields form_values as 'values, 'old_state', and
   *   'action' as the entity form operation i.e. bill, credit, default, delete,
   *    erase, sign, signoff etc.
   * @param string $eventName
   *   The machine name of the event.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   */
  public function onTransactionAction(TransactionSaveEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    if (\Drupal::State()->get('mcapi_mail_stop')) {// used for transaction imports
      return;
    }
    if ($this->mcapiSettings->get('mail_all')) {
      $tra = $event->getSubject();
      $payee_user = $tra->payee->entity->getOwner();
      $payer_user = $tra->payer->entity->getOwner();

      if ($payee_user->id() <> $this->currentUserId) {
        $this->basicNotify($payee_user, $tra);
      }
      elseif ($payer_user->id() <> $this->currentUserId) {
        $this->basicNotify($payer_user, $tra);
      }
    }
  }
  /**
   * Generate the basic transaction notification.
   * @param User $recipient
   * @param Transaction $transaction
   */
  private function basicNotify(User $recipient, Transaction $transaction) {
    \Drupal::service('plugin.manager.mail')->mail(
      'mcapi',
      'transaction_notification',
      $recipient->getEmail(),
      $recipient->getPreferredLangcode(),
      [
        'user' => $recipient,
        'xaction' => $transaction
      ]
    );
  }

}

==> mutual_credit-5.1.x-dev.tar.gz: mutual_credit/src/EventSubscriber/TransactionSubscriber.php <==
<?php

namespace Drupal\mcapi\EventSubscriber;

use Drupal\mcapi\Entity\Transaction;
use Drupal\mcapi\Event\McapiEvents;
use Drupal\mcapi\Event\TransactionSaveEvent;
use Drupal\mcapi\Event\TransactionAssembleEvent;
use Drupal\user\Entity\User;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Config;
use Drupal\Core\Session\AccountInterface;
use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Mail\MailManager;

/**
 * Subscriber with the two McapiEvents.
 */
class TransactionSubscriber implements EventSubscriberInterface {

  protected Config $mcapiSettings;
  protected int $currentUserId;
  protected State $state;
  protected MailManager $mailManager;

  function __construct(ConfigFactoryInterface $config_factory, AccountInterface $account, State $state, MailManager $mail_manager) {
    $this->mcapiSettings = $config_factory->get('mcapi.settings');
    $this->currentUserId = $account->id();
    $this->state = $state;
    $this->mailManager = $mail_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() : array{
    return [
      #McapiEvents::ASSEMBLE => ['makeChildren'],
      McapiEvents::ACTION => ['onTransactionAction']
    ];
  }

  /**
   * Example
   *
   * Payer pays a fee of 2 raw units to the wallet 1.
   *
   * @param TransactionAssembleEvent $event
   *   $event->getSubject() gives the transaction.
   * @param string $eventName
   *   Which is always mcapi_transaction.assemble.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   */
  public function makeChildren(TransactionAssembleEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    $event->getSubject()->createChild([
      'payer' => $original_transaction->payer->target_id,
      'payee' => 1,
      'quantity' => 2, // note this is the raw value. Same currency unless stated otherwise.
      'description' => $this->t('Payer fee to @wallet', ['@wallet' => Wallet::load(1)->name->value])
    ]);
  }

  /**
   * Mail the 2nd party to the transaction.
   *
   * @param TransactionSaveEvent $event
   *   $event->getSubject() gives the transaction.
   *   $event->getArguments() yields form_values as 'values, 'old_state', and
   *   'action' as the entity form operation i.e. bill, credit, default, delete,
   *    erase, sign, signoff etc.
   * @param string $eventName
   *   The machine name of the event.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   */
  public function onTransactionAction(TransactionSaveEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    if ($this->state->get('mcapi_mail_stop')) {// Used for transaction imports
      return;
    }
    if ($this->mcapiSettings->get('mail_all')) {
      $tra = $event->getSubject();
      $payee_user = $tra->payee->entity->getOwner();
      $payer_user = $tra->payer->entity->getOwner();

      if ($payee_user->id() <> $this->currentUserId) {
        $this->basicNotify($payee_user, $tra);
      }
      elseif ($payer_user->id() <> $this->currentUserId) {
        $this->basicNotify($payer_user, $tra);
      }
    }
  }

  /**
   * Generate the basic transaction notification.
   * @param User $recipient
   * @param Transaction $transaction
   */
  private function basicNotify(User $recipient, Transaction $transaction) {
    $this->mailManager->mail(
      'mcapi',
      'transaction_notification',
      $recipient->getEmail(),
      $recipient->getPreferredLangcode(),
      [
        'user' => $recipient,
        'xaction' => $transaction
      ]
    );
  }

}
